-- LOIH Cal -------------------------------------------------------------------
-------------------------------------------------------------------------------
local _G = getfenv(0)
local ADDON_NAME, ns = ... -- Addon name and private namespace
local L = ns.L -- Localization table


-- DEBUG ----------------------------------------------------------------------
-------------------------------------------------------------------------------
local DEBUGGING = false
--[===[@debug@
DEBUGGING = true
--@end-debug@]===]
local Debug = ns.Debug
local print_r = ns.print_r
local _normTime = ns._normTime


-- DEFAULTS AND UPVALUES ------------------------------------------------------
-------------------------------------------------------------------------------
-- API Upvalues
local table, ipairs, type, tostring, pairs = _G.table, _G.ipairs, _G.type, _G.tostring, _G.pairs
local next, string, select, setmetatable, rawset = _G.next, _G.string, _G.select, _G.setmetatable, _G.rawset

-- Constant Upvalues
local HIGHLIGHT_FONT_COLOR_CODE, FONT_COLOR_CODE_CLOSE = _G.HIGHLIGHT_FONT_COLOR_CODE, _G.FONT_COLOR_CODE_CLOSE
local RAID_CLASS_COLORS, RED_FONT_COLOR_CODE = _G.RAID_CLASS_COLORS, _G.RED_FONT_COLOR_CODE
local ORANGE_FONT_COLOR_CODE, GREEN_FONT_COLOR_CODE = _G.ORANGE_FONT_COLOR_CODE, _G.GREEN_FONT_COLOR_CODE
local NORMAL_FONT_COLOR_CODE, ITEM_QUALITY_COLORS = _G.NORMAL_FONT_COLOR_CODE, _G.ITEM_QUALITY_COLORS

-- Players name, Open event DB shortcut
local playerName, openEvent

-- DDM defaults
local DDDMItems, TDDMItems, defaultDifficulty, defaultLootMethod, defaultLootTreshold = {}, {}, 0, false, 0
--	1	"Normal"			2	"Heroic"				3	"10 Player"
--	4	"25 Player"			5	"10 Player (Heroic)"	6	"25 Player (Heroic)"
--	7	"Looking For Raid"	8	"Challenge Mode"		9	"40 Player"
--	10	nil					11	"Heroic Scenario"		12	"Normal Scenario"
--	13	nil					14	"Flexible"
--	14	"Normal Raid"		15	"Heroic Raid"			16	"Mythic Raid"			<-- WoD
for i = 14, 16 do -- Insert difficulties and their indexes to the table
	table.insert(DDDMItems, {name = GetDifficultyInfo(i), diff = i}) -- Localized
	if defaultDifficulty == 0 then -- Make the first default
		defaultDifficulty = i
	end
end
--	0 - Poor	1 - Common		2 - Uncommon	3 - Rare		<-- Gives error if you try to set treshold
--	4 - Epic	5 - Legendary	6 - Artifact	7 - Heirloom	<-- under 2, so we only take 2-5.
for i = 2, 5 do -- Insert treshold levels and their indexes to the table
	table.insert(TDDMItems, {name = _G["ITEM_QUALITY"..i.."_DESC"], tres = i}) -- Localized
	if defaultLootTreshold == 0 then -- Make the first default
		defaultLootTreshold = i
	end
end

-- Role table, Mass Invite table
local role, inviteTable = {}, {}

-- Create role sub-tables
local rolesList = { "Tanks", "Healers", "Melee", "Ranged", "Signup", "Standby" } -- Just to make updating all the Faux Scrollers easier
for _, v in ipairs(rolesList) do
	role[v] = {} -- Sub-tables to hold players for different roles
end

-- DB defaults
local DBdefaults = {
	global = {
		events = {
			['*'] = { -- Y
				['*'] = { --M
					['*'] = { --D
						['*'] = { -- T
							['*'] = { -- E
								['Players'] = {},
								['difficulty'] = defaultDifficulty,
								['method'] = defaultLootMethod,
								['treshold'] = defaultLootTreshold,
							},
						},
					},
				},
			},
		}
	}
}

-- Titlebar
local selected = false -- Selected titlebar
local TBC = { -- Titlebar colors
	["select"] = {1, 0, 0, 0.5}, -- Color when Selected
	["deselect"] = {0.3, 0.3, 0.3, 0.75} -- Color when Deselected
	-- Rest of the colors appear only once or so and can be hard coded.
	-- Plus these colors are the only ones I'm not 100% sure about yet.
}

-- Round-Function, Frames and FrameFunctions local-scoped outside of the do-blocks
local _round
local Main, Tanks, Healers, Melee, Ranged, Signup, Standby, DDDM, TDDM, MLCB, MIB
local _titleOnClick, _slotOnClick, _updateScrollBar
local _DDDMOnSelect, _TDDMOnSelect, _DDDMInitialize, _TDDMInitialize, _MLCBClick, _MIBClick


-- EVENTS ---------------------------------------------------------------------
-------------------------------------------------------------------------------
do -- Events block
	ns = CreateFrame("Frame") -- Event frame
	ns:SetScript("OnEvent", function(self, event, ...) return self[event](self, ...) end)
	ns:RegisterEvent("ADDON_LOADED")

	function ns:ADDON_LOADED(addon) -- Initialize
		if addon ~= ADDON_NAME then return end

		if DEBUGGING then Debug("%s E: Addon Loaded.", _normTime()) end -- Debug

		-- Initialize DB
		self.db = LibStub("AceDB-3.0"):New(ADDON_NAME.."DB", DBdefaults, true)

		if self.db then
			if DEBUGGING then Debug("DB %sDB Loaded.", ADDON_NAME) end -- Debug
		end

		if type(self.db.global.events) ~= "table" then
			self.db.global.events = {}

			if DEBUGGING then Debug("Error loading Global events DB, type: \"%s\".", tostring(type(self.db.global.events))) end -- Debug
		else
			if DEBUGGING then Debug("Found Gloval events DB.") end -- Debug
		end

		-- Initialize done, wait for "Enable"
		if IsLoggedIn() then
			self:PLAYER_LOGIN()
		else
			self:RegisterEvent("PLAYER_LOGIN")
		end
	end

	function ns:PLAYER_LOGIN() -- "Enable"
		self:UnregisterEvent("PLAYER_LOGIN")

		self:RegisterEvent("CALENDAR_EVENT_ALARM")
		self:RegisterEvent("CALENDAR_OPEN_EVENT")
		self:RegisterEvent("CALENDAR_UPDATE_EVENT")
		self:RegisterEvent("CALENDAR_UPDATE_INVITE_LIST")

		-- First time Hooking without AceHook, hope this is done right...
		CalendarViewEventFrame:HookScript("OnHide", function(self) -- ViewEventFrame OnHide Hook
			Main:Hide()
		end)
		CalendarCreateEventFrame:HookScript("OnHide", function(self) -- CreateEventFrame OnHide Hook
			Main:Hide()
		end)

		playerName = UnitName("player")

		self:PurgeDB()
		self:CreateUI()

		if DEBUGGING then Debug("%s Loaded.", ADDON_NAME) end -- Debug
	end

	function ns:CALENDAR_EVENT_ALARM(event, ...) -- 15mins to event, work on this after everything works
		local title, hour, minute = ...

		if DEBUGGING then Debug("%s E: Event Alarm: \"%s\", \"%s\", %i, %i.", _normTime(), tostring(event), title, hour, minute) end -- Debug
	end

	function ns:CALENDAR_OPEN_EVENT(type)
		if DEBUGGING then Debug("%s E: Open Event: \"%s\".", _normTime(), tostring(type or "!...")) end -- Debug
		
		if type ~= "PLAYER" and type ~= "GUILD_EVENT" then return end

		if CalendarViewEventFrame:IsShown() then
			Main:SetPoint("TOPLEFT", CalendarViewEventFrame, "TOPRIGHT", 26, 0)
			Main:Show()
		else
			Main:SetPoint("TOPLEFT", CalendarCreateEventFrame, "TOPRIGHT", 26, 0)
			Main:Show()
		end

		if selected then -- Disable selected group on event change
			selected.t:SetTexture(TBC.deselect[1], TBC.deselect[2], TBC.deselect[3], TBC.deselect[4])
			selected = false 
		end

		-- Calendar might be busy waiting server for event info, let's wait...
		local timer = 0
		self:SetScript("OnUpdate", function(self, elapsed)
			timer = timer + elapsed
			while timer >= 0.25 do
				local _, namesReady = CalendarEventGetNumInvites() -- This should prevent updatePlayers running before getting all the names from server
				if not CalendarIsActionPending() and namesReady then -- Can I have moment of your time?
					self:SetScript("OnUpdate", nil)
					self:updatePlayers() -- Update DB and role views
				end
				timer = timer - 0.25
			end
		end)
	end

	function ns:GROUP_ROSTER_UPDATE()
		if DEBUGGING then Debug("%s E: Group Roster Update.", _normTime()) end -- Debug

		if GetNumGroupMembers() > 0 and not IsInRaid() then -- In group but not in raid
			ConvertToRaid()
		else -- Magic is done
			self:UnregisterEvent("GROUP_ROSTER_UPDATE")
		end
	end

	function ns:CALENDAR_UPDATE_EVENT()
		if DEBUGGING then Debug("%s E: Update Event.", _normTime()) end -- Debug

		self:updatePlayers() -- Update DB and role views
	end

	function ns:CALENDAR_UPDATE_INVITE_LIST()
		if DEBUGGING then Debug("%s E: Update Invite List.", _normTime()) end -- Debug

		self:updatePlayers() -- Update DB and role views
	end
end -- Events block


-- DATABASE -------------------------------------------------------------------
-------------------------------------------------------------------------------
do -- DB functions
	function ns:PurgeDB() -- Remove old stuff (last year and last month) from DB
		if DEBUGGING then Debug("%s PurgeDB.", _normTime()) end -- Debug

		local events = self.db.global.events
		local _, month, day, year = CalendarGetDate()
		if events and year and month then -- Got DB and current date info
			for k, v in pairs(events) do
				if k < year then
					if DEBUGGING then Debug("Remove year %i, current year %i.", k, year) end -- Debug

					events[k] = nil
				else
					if DEBUGGING then Debug("Keep year %i, current year %i.", k, year) end -- Debug

					for i, _ in pairs(v) do
						if i < month then
							if DEBUGGING then Debug("Remove month %i, current month %i.", i, month) end -- Debug

							events[k][i] = nil
						else
							if DEBUGGING then Debug("Keep month %i, current month %i.", i, month) end -- Debug
						end
					end	
				end
			end
		end
	end
end -- DB functions


-- FUNCTIONS ------------------------------------------------------------------
-------------------------------------------------------------------------------
do -- Functions block
	-- Round function, copied from http://lua-users.org/wiki/SimpleRound
	function _round(num, idp)
		local mult = 10^(idp or 0)
		return math.floor(num * mult + 0.5) / mult
	end

	-- Returns players index in Events invite list
	local function _getIndex(indexName)
		for i = 1, CalendarEventGetNumInvites() do
			local name = CalendarEventGetInvite(i)
			if name == indexName then
				return i
			end
		end
		return false
	end

	-- Set up raid difficulty and loot settings
	local function _setRaidUp()
		if DEBUGGING then Debug("%s _setRaidUp.", _normTime()) end -- Debug

		if GetRaidDifficultyID() ~= openEvent.difficulty then -- Set raid difficulty
			SetRaidDifficultyID(openEvent.difficulty)
		end

		if openEvent.method and GetLootMethod() ~= "master" then -- Checked for ML, but no ML on yet
			SetLootMethod("master")
		elseif GetLootThreshold() ~= openEvent.treshold then -- Wrong treshold. Reason why this is checked on elseif isntead of if is explained on comment below
			--Calling this while the loot method is changing will revert to the original loot method. To account for this I simply set the loot threshold 1 second after calling API_SetLootMethod
			SetLootThreshold(openEvent.treshold)
		end
	end

	-- Update text on "Statusbar" on bottom
	local numSignup = 0
	local function _bottomText()
		if DEBUGGING then Debug("%s _bottomText.", _normTime()) end -- Debug

		local numTanks, numHealers, numMelee, numRanged, numStandby = #role["Tanks"], #role["Healers"], #role["Melee"], #role["Ranged"], #role["Standby"] -- Count players in different roles
		
		local space = 1
		Main.bottom.s:SetText(L["Signups"].." "..HIGHLIGHT_FONT_COLOR_CODE..numSignup..FONT_COLOR_CODE_CLOSE..string.rep(" ", space).." "..L["Tanks"].." "..HIGHLIGHT_FONT_COLOR_CODE..numTanks..FONT_COLOR_CODE_CLOSE..string.rep(" ", space).." "..L["Healers"].." "..HIGHLIGHT_FONT_COLOR_CODE..numHealers..FONT_COLOR_CODE_CLOSE..string.rep(" ", space).." "..L["DPS"].." "..HIGHLIGHT_FONT_COLOR_CODE..numMelee + numRanged..FONT_COLOR_CODE_CLOSE.." ("..HIGHLIGHT_FONT_COLOR_CODE..numMelee..FONT_COLOR_CODE_CLOSE.." + "..HIGHLIGHT_FONT_COLOR_CODE..numRanged..FONT_COLOR_CODE_CLOSE..") "..string.rep(" ", space).." "..L["Standbys"].." "..HIGHLIGHT_FONT_COLOR_CODE..numStandby..FONT_COLOR_CODE_CLOSE)

		while Main.bottom.s:GetStringWidth() < 309 do -- Expand the line by maxing out the space between text, and make it one step too long (Max width 318, but we want 5px marginal per side)
			space = space + 1
			Main.bottom.s:SetText(L["Signups"].." "..HIGHLIGHT_FONT_COLOR_CODE..numSignup..FONT_COLOR_CODE_CLOSE..string.rep(" ", space).." "..L["Tanks"].." "..HIGHLIGHT_FONT_COLOR_CODE..numTanks..FONT_COLOR_CODE_CLOSE..string.rep(" ", space).." "..L["Healers"].." "..HIGHLIGHT_FONT_COLOR_CODE..numHealers..FONT_COLOR_CODE_CLOSE..string.rep(" ", space).." "..L["DPS"].." "..HIGHLIGHT_FONT_COLOR_CODE..numMelee + numRanged..FONT_COLOR_CODE_CLOSE.." ("..HIGHLIGHT_FONT_COLOR_CODE..numMelee..FONT_COLOR_CODE_CLOSE.." + "..HIGHLIGHT_FONT_COLOR_CODE..numRanged..FONT_COLOR_CODE_CLOSE..") "..string.rep(" ", space).." "..L["Standbys"].." "..HIGHLIGHT_FONT_COLOR_CODE..numStandby..FONT_COLOR_CODE_CLOSE)
		end

		if space > 0 then -- Now the line is too long, so reduce space by 1, but don't make smaller than 0
			space = space - 1
		end

		Main.bottom.s:SetText(L["Signups"].." "..HIGHLIGHT_FONT_COLOR_CODE..numSignup..FONT_COLOR_CODE_CLOSE..string.rep(" ", space).." "..L["Tanks"].." "..HIGHLIGHT_FONT_COLOR_CODE..numTanks..FONT_COLOR_CODE_CLOSE..string.rep(" ", space).." "..L["Healers"].." "..HIGHLIGHT_FONT_COLOR_CODE..numHealers..FONT_COLOR_CODE_CLOSE..string.rep(" ", space).." "..L["DPS"].." "..HIGHLIGHT_FONT_COLOR_CODE..numMelee + numRanged..FONT_COLOR_CODE_CLOSE.." ("..HIGHLIGHT_FONT_COLOR_CODE..numMelee..FONT_COLOR_CODE_CLOSE.." + "..HIGHLIGHT_FONT_COLOR_CODE..numRanged..FONT_COLOR_CODE_CLOSE..") "..string.rep(" ", space).." "..L["Standbys"].." "..HIGHLIGHT_FONT_COLOR_CODE..numStandby..FONT_COLOR_CODE_CLOSE)
	end

	-- Get Players from Calendar and DB
	function ns:updatePlayers()
		if DEBUGGING then Debug("%s updatePlayers.", _normTime()) end -- Debug

		local title, _, creator, _, _, _, textureIndex, _, month, day, year, hour, minute = CalendarGetEventInfo()

		if title and creator and textureIndex and month and day and year and hour and minute then
			for _, v in ipairs(rolesList) do -- Wipe Roles
				wipe(role[v])
			end
			openEvent = self.db.global.events[year][month][day][hour..minute][textureIndex.."-"..creator]

			if type(openEvent) ~= "table" then
				if DEBUGGING then Debug("openEvent type missmatch: \"%s\"... Abort DB stuff.", type(openEvent or "!...")) end -- Debug
				return
			end

			if not next(openEvent["Players"]) then -- New event, create one
				if DEBUGGING then Debug("Created new DB entry %i-%s.", textureIndex, creator) end -- Debug

				openEvent.title = title -- Title for event
				Main.title.s:SetText(ADDON_NAME.." - "..title)
				openEvent.difficulty = defaultDifficulty -- Default
				openEvent.method = defaultLootMethod -- Default
				openEvent.treshold = defaultLootTreshold -- Default
				DDDM:SetSelectedID(openEvent.difficulty-13) -- List items 1-4, Difficulty items 14-16
				TDDM:SetSelectedID(openEvent.treshold-1) -- List items 1-4, Treshold items 2-5
				MLCB:SetChecked(openEvent.method)

				for i = 1, CalendarEventGetNumInvites() do -- Insert names into table
					local name, level, _, classFileName, inviteStatus, modStatus = CalendarEventGetInvite(i)

					openEvent["Players"][name] = {name = name, class = classFileName, level = level, status = inviteStatus, role = "", moderator = modStatus}
				end

				-- Now we have player info, let's see if we can enable the MIB
				if openEvent and openEvent["Players"] ~= nil and openEvent["Players"] ~= "" and openEvent["Players"][playerName] ~= nil and openEvent["Players"][playerName] ~= "" and -- Fixing the nil bug
					openEvent["Players"][playerName]["moderator"] ~= nil and openEvent["Players"][playerName]["moderator"] ~= "" and -- More nil bug fixes to be sure
					(openEvent["Players"][playerName]["moderator"] == "CREATOR" or openEvent["Players"][playerName]["moderator"] == "MODERATOR") then
					MIB:Enable()
				else
					MIB:Disable()
				end
			else -- Old event, update it
				if DEBUGGING then Debug("Found DB entry %i-%s, updating.", textureIndex, creator) end -- Debug

				openEvent.title = title -- Update title
				Main.title.s:SetText(ADDON_NAME.." - "..title)
				DDDM:SetSelectedID(openEvent.difficulty-13)--6.0 2
				TDDM:SetSelectedID(openEvent.treshold-1)
				MLCB:SetChecked(openEvent.method)

				for k, _ in pairs(openEvent["Players"]) do -- Delete removed names (maybe someone ragequit or changed guild)
					local found = false
					for i = 1, CalendarEventGetNumInvites() do
						found = false
						local name = CalendarEventGetInvite(i)
						if name and name ~= "" and openEvent["Players"][k]["name"] == name then
							found = true -- Mr "name" didn't ragequit
							break
						end
					end
					if not found then -- Aha! The usual suspect, remove old name
						--table.remove(openEvent["Players"][k])
						if openEvent["Players"][k]["name"] == playerName then -- If you remove yourself from event
							if DEBUGGING then Debug("I removed myself from event %i-%s.", textureIndex, creator) end -- Debug

							openEvent = nil -- Remove the whole event
							return
						end

						if DEBUGGING then Debug("Removed \"%s\" from event %i-%s.", tostring(openEvent["Players"][k]["name"]), textureIndex, creator) end -- Debug

						openEvent["Players"][k] = nil
					end
				end

				for i = 1, CalendarEventGetNumInvites() do -- Check for new friends
					local name, level, _, classFileName, inviteStatus, modStatus = CalendarEventGetInvite(i)
					if name and name ~= "" and not openEvent["Players"][name] then -- Insert new name if found
						openEvent["Players"][name] = {name = name, class = classFileName, level = level, status = inviteStatus, role = "", moderator = modStatus}
					elseif name and name ~= "" and openEvent["Players"][name] then -- Found old friend instead, let's update his/her level, inviteStatus and modStatus
						openEvent["Players"][name]["level"] = level -- Level ups?
						openEvent["Players"][name]["status"] = inviteStatus -- Did you accept the invitation or were you confirmed since we last saw?
						openEvent["Players"][name]["moderator"] = modStatus -- Ranked up to MODERATOR of event?
					else -- This shouldn't happen
						if DEBUGGING then Debug("??? Found unnamed character \"%s\" from calendar for event %i-%s.", tostring(name), textureIndex, creator) end -- Debug
					end
				end
				-- List is now up to date

				-- Now we have the updated player info, let's see if we can enable the Mass Invite Button
				if openEvent["Players"][playerName] and (openEvent["Players"][playerName]["moderator"] == "CREATOR" or openEvent["Players"][playerName]["moderator"] == "MODERATOR") then
					MIB:Enable()
				else
					MIB:Disable()
				end
			end

			-- Players in event DB
			numSignup = 0
			for k, _ in pairs(openEvent["Players"]) do -- Throw them into role slots
				if openEvent["Players"][k]["role"] == "" or openEvent["Players"][k]["role"] == nil then
					openEvent["Players"][k]["role"] = "Signup"
				end
				if openEvent["Players"][k]["status"] == 3 or openEvent["Players"][k]["status"] == 5 or openEvent["Players"][k]["status"] == 9 then -- Red or Tentative
					openEvent["Players"][k]["role"] = "Signup"
				elseif openEvent["Players"][k]["status"] == 6 then -- Standby
					openEvent["Players"][k]["role"] = "Standby"
				end

				local found = false
				for i = #role[openEvent["Players"][k]["role"]], 1, -1 do
					found = false
					if openEvent["Players"][k]["name"] == role[openEvent["Players"][k]["role"]][i]["name"] then
						found = true
						break
					end
				end

				if not found then
					table.insert(role[openEvent["Players"][k]["role"]], openEvent["Players"][k])
				end

				if openEvent["Players"][k]["status"] == 2 or openEvent["Players"][k]["status"] == 4 or openEvent["Players"][k]["status"] == 7 then -- Count signups (Accepted, Confirmed and Signed Ups)
					numSignup = numSignup + 1
				end
			end
			for _, v in ipairs(rolesList) do -- Update roles
				_updateScrollBar(_G[v], role[v])
			end

			_bottomText() -- Update text on "Statusbar" on bottom
		end
	end

	-- Transaction players between role-tables
	function ns:_transaction(sparent, source, value, tparent, target)
		if DEBUGGING then Debug("%s _transaction: %s (%s) - %i - %s (%s)", _normTime(), source, tostring(sparent:GetName()), value, target, tostring(tparent:GetName())) end -- Debug

		playerName = playerName or UnitName("player")

		role[source][value]["role"] = target
		openEvent["Players"][role[source][value]["name"]]["role"] = target
		if target == "Standby" then -- Change status to Standby if moved to the group
			role[source][value]["status"] = 6
			openEvent["Players"][role[source][value]["name"]]["status"] = 6
			if _getIndex(role[source][value]["name"]) and openEvent["Players"][playerName] ~= nil and (openEvent["Players"][playerName]["moderator"] == "CREATOR" or openEvent["Players"][playerName]["moderator"] == "MODERATOR") then -- Put player on Standby, but only if you have the rights
				local index = _getIndex(role[source][value]["name"])

				CalendarEventSetStatus(index, 6) -- The real stuff
			end
		elseif target ~= "Signup" then -- Change status to Confirmed if moved to other than Signup group
			role[source][value]["status"] = 4
			openEvent["Players"][role[source][value]["name"]]["status"] = 4
			if _getIndex(role[source][value]["name"]) and openEvent["Players"][playerName] ~= nil and (openEvent["Players"][playerName]["moderator"] == "CREATOR" or openEvent["Players"][playerName]["moderator"] == "MODERATOR") then -- Confirm player, but only if you have the rights
				local index = _getIndex(role[source][value]["name"])

				CalendarEventSetStatus(index, 4) -- The real stuff
			end
		end

		if DEBUGGING then Debug("Transaction Role: %s / %s.", role[source][value]["role"], openEvent["Players"][role[source][value]["name"]]["role"]) end -- Debug

		table.insert(role[target], role[source][value])
		table.remove(role[source], value)

		_updateScrollBar(tparent, role[target])
		_updateScrollBar(sparent, role[source])

		_bottomText() -- Update text on "Statusbar" on bottom
	end

	-- Mass Invite people to the raid
	local invWhisper = L["Diudiu darlings! Raid event \"%s\" is about to start, please accept this invite. (This message was sent by %s-addon)"]
	function ns:massInvite()
		if DEBUGGING then Debug("%s massInvite.", _normTime()) end -- Debug
	 
		if #inviteTable == 0 then return end

		playerName = playerName or UnitName("player")
		self:RegisterEvent("GROUP_ROSTER_UPDATE")

		local timer = 0
		self:SetScript("OnUpdate", function(self, elapsed)
			timer = timer + elapsed
			while timer >= 2 do -- Give this enough time to do the changes
				if #inviteTable == 0 then
					if DEBUGGING then Debug("Mass Invite: inviteTable empty.") end -- Debug

					self:SetScript("OnUpdate", nil)
					self:UnregisterEvent("GROUP_ROSTER_UPDATE")
					break -- To prevent InviteUnit(nil)
				end
				if IsInRaid() then
					self:UnregisterEvent("GROUP_ROSTER_UPDATE")
					_setRaidUp()
				end
				
				for i = 1, GetNumGroupMembers() do
					local name = GetRaidRosterInfo(i)
					for k, _ in ipairs(inviteTable) do
						if name == inviteTable[k] then -- Already in group
							table.remove(inviteTable, k) -- Not going to invite you again
						end
					end
				end

				if #inviteTable > 0 then -- Still someone in the table
					if DEBUGGING then Debug("Mass Invite: Inviting \"%s\".\n%s.", tostring(inviteTable[1]), string.format(invWhisper, openEvent.title, ADDON_NAME)) end -- Debug

					SendChatMessage(string.format(invWhisper, openEvent.title, ADDON_NAME), "WHISPER", nil, inviteTable[1]) -- 1. Whisper
					InviteUnit(inviteTable[1]) -- 2. Invite
					table.remove(inviteTable, 1) -- 3. Remove from queue
				end
				timer = timer - 2
			end
		end)
	end
end -- Functions block


-- FRAMEFUNCTIONS -------------------------------------------------------------
-------------------------------------------------------------------------------
do -- Framefunction block
	-- Return class-color table or colorstring
	local function _colorMe(input)
		if type(input) == "string" then -- Class
			return RAID_CLASS_COLORS[input]
		else -- Numeric
			if input == 3 or input == 5 then -- Red
				return RED_FONT_COLOR_CODE
			elseif input == 6 or input == 9 then -- Orange
				return ORANGE_FONT_COLOR_CODE
			elseif input == 2 or input == 4 or input == 7 then -- Green
				return GREEN_FONT_COLOR_CODE
			else -- (1, 8) Normal text
				return NORMAL_FONT_COLOR_CODE
			end
		end
	end

	-- Endspace calculated
	local spaceCalc
	local function _spaceMe(element) -- Calculate how many spaces we have to use in the end of the line to prevent it going behind Scrollbar
		if not spaceCalc then -- No previous endSpace
			local space = 1
			element:SetText(string.rep(" ", space))
			while element:GetStringWidth() <= 21 do -- 16px Scrollbar + 5+px margin
				space = space + 1
				element:SetText(string.rep(" ", space))
			end
			spaceCalc = space
			return spaceCalc
		else -- Previous endSpace exists, we shouldn't hit this, but added for safety
			return spaceCalc
		end
	end

	-- Selected title
	function _titleOnClick(self) -- Select and highlight selected Titlebar
		if DEBUGGING then Debug("%s _titleOnClick: \"%s\".", _normTime(), self:GetName()) end -- Debug

		if not selected then -- Nothing selected, selecting pressed group
			selected = self
			self.t:SetTexture(TBC.select[1], TBC.select[2], TBC.select[3], TBC.select[4])
		elseif selected ~= self then -- Deselect previous group and select pressed group
			selected.t:SetTexture(TBC.deselect[1], TBC.deselect[2], TBC.deselect[3], TBC.deselect[4])
			selected = self
			self.t:SetTexture(TBC.select[1], TBC.select[2], TBC.select[3], TBC.select[4])
		else -- Deselcting already selected group
			selected = false
			self.t:SetTexture(TBC.deselect[1], TBC.deselect[2], TBC.deselect[3], TBC.deselect[4])
		end
	end

	-- Slot Button OnClick
	function _slotOnClick(self)
		if DEBUGGING then Debug("%s _slotOnClick: \"%s\" (%i).", _normTime(), self:GetName(), self.value) end -- Debug

		if selected and selected:GetParent() ~= self:GetParent() then
			local source = self:GetParent()
			source.name = self:GetParent():GetName()
			local target = selected:GetParent()
			target.name = selected:GetParent():GetName()
			local value = self.value

			ns:_transaction(source, source.name, value, target, target.name) -- Pointer, Name, Row#, Pointer, Name
		end
	end

	-- Faux Scroller
	local calStatus = { -- Invite statuses
		[1] = L["Inv"], --CALENDAR_INVITESTATUS_INVITED
		[2] = L["Acc"], --CALENDAR_INVITESTATUS_ACCEPTED
		[3] = L["Dec"], --CALENDAR_INVITESTATUS_DECLINED
		[4] = L["Con"], --CALENDAR_INVITESTATUS_CONFIRMED
		[5] = L["Out"], --CALENDAR_INVITESTATUS_OUT
		[6] = L["Sta"], --CALENDAR_INVITESTATUS_STANDBY
		[7] = L["Sig"], --CALENDAR_INVITESTATUS_SIGNEDUP
		[8] = L["Not"], --CALENDAR_INVITESTATUS_NOT_SIGNEDUP
		[9] = L["Ten"] --CALENDAR_INVITESTATUS_TENTATIVE
	}
	function _updateScrollBar(self, tbl) -- Update Slot Buttons
		local maxValue = #tbl
		local slots = 10
		if self.name == "Tanks" or self.name == "Healers" or self.name == "Standby" then
			slots = 6
		elseif self.name == nil then -- If we update from updatePlayers()
			if self:GetName() == "Tanks" or self:GetName() == "Healers" or self:GetName() == "Standby" then
				slots = 6
			end
		end
		FauxScrollFrame_Update(self.scrollBar, maxValue, slots, 15) -- Scrollbar, max items on scrollbar, max visible items on scrollbar and height of item on scrollbar
		local offset = FauxScrollFrame_GetOffset(self.scrollBar)
		for i = 1, slots do
			local value = i + offset
			if value <= maxValue then
				local row = self.rows[i]

				-- Draw the player buttons
				row.value = value
				row.name = tbl[value]["name"]
				if row.name:match("^([^%-]+)%-(.*)$") then -- Player is from another realm
					row.name = select(1, row.name:match("^([^%-]+)%-(.*)$")).." (*)" -- Shorten realm name into (*)
				end
				row.level = tbl[value]["level"]
				row.status = tbl[value]["status"]
				row.class = tbl[value]["class"]

				row.cc = _colorMe(row.class) or {r = 1, g = 1, b = 1, colorStr = HIGHLIGHT_FONT_COLOR_CODE}
				row.sc = _colorMe(row.status) or HIGHLIGHT_FONT_COLOR_CODE

				local space = 1
				local endSpace = spaceCalc or _spaceMe(row.s)
				
				-- Leave ~5px margin (endSpace/4), Status --Space-- Name --Space-- Level, ~5px margin + 16px Scrollbar (endSpace)
				row.s:SetText(string.rep(" ", _round(endSpace/4))..row.sc..calStatus[row.status].."|r"..string.rep(" ", space).."|c"..row.cc.colorStr..row.name.."|r"..string.rep(" ", space)..row.level..string.rep(" ", endSpace))

				while row.s:GetStringWidth() < 151 do -- Expand the line by maxing out the space between text, and make it one step too long (Max width 150, but we want 5px marginal per side)
					space = space + 1
					row.s:SetText(string.rep(" ", _round(endSpace/4))..row.sc..calStatus[row.status].."|r"..string.rep(" ", space).."|c"..row.cc.colorStr..row.name.."|r"..string.rep(" ", space)..row.level..string.rep(" ", endSpace))
				end

				if space > 0 then -- Now the line is too long, so reduce space by 1, but don't make smaller than 0
					space = space - 1
				end

				row.s:SetText(string.rep(" ", _round(endSpace/4))..row.sc..calStatus[row.status].."|r"..string.rep(" ", space).."|c"..row.cc.colorStr..row.name.."|r"..string.rep(" ", space)..row.level..string.rep(" ", endSpace))

				row.t:SetTexture(row.cc.r, row.cc.g, row.cc.b, 0.35)
				row:Show()
				
				-- Live Version
				if row.status == 2 or row.status == 4 or row.status == 7 or row.status == 6 then -- Green people + Standby
					row:Enable()
				else -- If you really want to sign these people, change their status in calendar
					row:Disable()
					row.t:SetTexture(0.25, 0.25, 0.25, 0.25) -- Gray was too light (0.5, 0.5, 0.5)
				end

			else
				self.rows[i]:Hide()
			end
		end
	end

	-- Dropdown Menu
	function _DDDMOnSelect(self, arg1, arg2, checked) -- On DDM item select function
		if DEBUGGING then Debug("%s _DDDMOnSelect: ID %i, Value %i, Checked \"%s\".", _normTime(), self:GetID(), self.value, tostring(checked)) end -- Debug
			
		if not checked then
			DDDM:SetSelectedID(self:GetID())
			openEvent.difficulty = self.value or defaultDifficulty -- To the DBase or default (3)
		end
	end

	function _TDDMOnSelect(self, arg1, arg2, checked) -- On DDM item select function
		if DEBUGGING then Debug("%s _TDDMOnSelect: ID %i, Value %i, Checked \"%s\".", _normTime(), self:GetID(), self.value, tostring(checked)) end -- Debug
			
		if not checked then
			TDDM:SetSelectedID(self:GetID())
			openEvent.treshold = self.value or defaultLootTreshold -- To the DBase or default (2)
		end
	end

	local info = {} -- Create only once
	function _DDDMInitialize() -- DDDM Initialize
		if DEBUGGING then Debug("%s _DDDMInitialize.", _normTime()) end -- Debug

		--local info = UIDropDownMenu_CreateInfo() -- Source of possible taints?
		wipe(info)
		info.func = _DDDMOnSelect

		for i = 1, #DDDMItems do
			local name = DDDMItems[i]["name"]
			local diff = DDDMItems[i]["diff"]
			info.text = name
			info.value = diff

			if not openEvent then
				info.checked = diff == DDDM.selected
			else
				info.checked = diff == openEvent.difficulty
			end

			UIDropDownMenu_AddButton(info)
		end
	end

	function _TDDMInitialize() -- TDDM Initialize
		if DEBUGGING then Debug("%s _TDDMInitialize.", _normTime()) end -- Debug

		wipe(info)
		info.func = _TDDMOnSelect

		for i = 1, #TDDMItems do
			local name = TDDMItems[i]["name"]
			local tres = TDDMItems[i]["tres"]
			info.text = name
			info.value = tres
			info.colorCode = ITEM_QUALITY_COLORS[tres].hex

			if not openEvent then
				info.checked = tres == TDDM.selected
			else
				info.checked = tres == openEvent.treshold
			end

			UIDropDownMenu_AddButton(info)
		end
	end

	-- Master Looter Checkbox
	function _MLCBClick() -- On Click function
		if DEBUGGING then Debug("%s _MLCBClick: \"%s\".", _normTime(), tostring(MLCB:GetChecked())) end -- Debug

		if MLCB:GetChecked() then
			openEvent.method = true
		else
			openEvent.method = false
		end
	end

	-- Mass Invite Button
	function _MIBClick() -- On click function
		if DEBUGGING then Debug("%s _MIBClick.", _normTime()) end -- Debug

		-- updatePlayers() checks this, but let's check again if the button is accidently still enabled
		if openEvent["Players"][playerName]["moderator"] == "CREATOR" or openEvent["Players"][playerName]["moderator"] == "MODERATOR" then
			wipe(inviteTable)
			for k, _ in pairs(openEvent["Players"]) do
				if openEvent["Players"][k]["status"] == 4 and openEvent["Players"][k]["role"] ~= "Signup" then -- Add only Confirmed people with role assigned to them
					if openEvent["Players"][k]["name"] ~= playerName then -- Don't add me, I'm already here
						table.insert(inviteTable, openEvent["Players"][k]["name"])
					end
				end
			end

			ns:massInvite() -- Send info to invite function
		end
	end
end -- Framefunction block


-- FACTORY --------------------------------------------------------------------
-------------------------------------------------------------------------------
do -- Factory block
	-- Background for frames
	local bg = {
		bgFile = 'Interface\\DialogFrame\\UI-DialogBox-Background',
		edgeFile = 'Interface\\ChatFrame\\ChatFrameBackground',
		edgeSize = 1,
		insets = {left = 1, right = 1, top = 1, bottom = 1},
		tile = true,
	}

	-- Frame Factory
	function ns:_frameFactory(type, name, parent, width, height, template) -- Creates frames and buttons
		local f = CreateFrame(type, name, parent, template)
		if width and height then
			f:SetSize(width, height)
		end
		if type == "Frame" then
			f:SetBackdrop(bg)
			f:SetBackdropBorderColor(0.65, 0.65, 0.65, 1)
		end

		return f
	end

	-- Title Factory
	function ns:_titleFactory(parent, offset, text) -- Creates titles for frames
		local f = CreateFrame("Button", "$parentTitle", parent)
		f:SetSize(152, 15)
		f:SetPoint("TOP", 0, offset)

		local s = f:CreateFontString(nil,"OVERLAY", "GameFontNormal")
		s:SetPoint("CENTER")
		s:SetText(text)
		f.s = s
		if text ~= " " then -- Don't create texture for Bottombar
			local t = f:CreateTexture()
			t:SetParent(f)
			t:SetAllPoints(f)
			t:SetTexture(TBC.deselect[1], TBC.deselect[2], TBC.deselect[3], TBC.deselect[4])
			f.t = t
		end

		f:SetScript("OnClick", _titleOnClick)

		return f
	end

	-- Icon Factory
	function ns:_iconFactory(parent, role) -- Creates role icons
		local f = CreateFrame("Button", "$parentIcon", parent)
		f:SetSize(16, 16)
		f:EnableMouse(false) -- Click-through
		if role == "TITLE" then
			f:SetScale(8) -- 128x128
			f:SetFrameLevel(parent:GetParent():GetFrameLevel()) -- Push Logo behind other frames
			f:SetPoint("CENTER")
		elseif role == "DIFFICULTY" or role == "LOOT" or role == "INVITE" then
			f:SetPoint("RIGHT", parent.s, "LEFT", -2, 0)
		else
			f:SetPoint("RIGHT", parent.s, "LEFT", 0, 1)
		end
	
		local t = f:CreateTexture()
		t:SetParent(f)
		t:SetAllPoints(f)
		if role == "TITLE" then -- Lion Head -icon for main window title
			t:SetTexture("Interface\\Timer\\Alliance-Logo")
		elseif role == "TANK" then
			t:SetTexture("Interface\\LFGFrame\\LFGRole")
			t:SetTexCoord(0.5, 0.75, 0, 1)
		elseif role == "HEALER" then
			t:SetTexture("Interface\\LFGFrame\\LFGRole")
			t:SetTexCoord(0.75, 1, 0, 1)
		elseif role == "DPS" then
			t:SetTexture("Interface\\LFGFrame\\LFGRole")
			t:SetTexCoord(0.25, 0.5, 0, 1)
		elseif role == "SIGNUP" then
			t:SetTexture("Interface\\Minimap\\ObjectIcons")
			t:SetTexCoord(0.5, 0.625, 0.125, 0.25)
		elseif role == "STANDBY" then
			t:SetTexture("Interface\\Minimap\\ObjectIcons")
			t:SetTexCoord(0.625, 0.75, 0.5, 0.625)
		elseif role == "DIFFICULTY" then
			t:SetTexture("Interface\\Minimap\\ObjectIcons")
			t:SetTexCoord(0.625, 0.75, 0.375, 0.5)
		elseif role == "LOOT" then
			--t:SetTexture("Interface\\GroupFrame\\UI-Group-MasterLooter")
			--t:SetTexCoord(0, 1, 0, 1)
			t:SetTexture("Interface\\Minimap\\ObjectIcons")
			t:SetTexCoord(0.125, 0.25, 0.25, 0.375)
		elseif role == "INVITE" then
			t:SetTexture("Interface\\Buttons\\UI-GuildButton-MOTD-Up")
			t:SetTexCoord(0, 1, 0, 1)
		end
		f.t = t

		return f
	end

	-- Slot Factory
	function ns:_slotFactory(number, parent, ...)
		local f = CreateFrame("Button", "$parentRow"..number, parent)
		f:SetSize(150, 15)
		f:SetNormalFontObject(GameFontNormal)
		if number == 1 then
			f:SetPoint("TOP", parent, "TOP", 0, -1) -- 1px Marginal on top of the parent
		else
			f:SetPoint("TOP", parent.rows[number-1], "BOTTOM")
		end

		local s = f:CreateFontString(nil,"OVERLAY", "GameFontNormalLeft")
		s:SetPoint("CENTER")
		f.s = s

		local t = f:CreateTexture()
		t:SetParent(f)
		t:SetAllPoints(f)
		t:SetTexture(1, 1, 1, 0.25)
		f.t = t

		f:SetScript("OnClick", _slotOnClick)

		return f
	end

	-- Faux Scroller ScrollBar Factory
	function ns:_scrollBarFactory(parent, tbl)
		local f = CreateFrame("ScrollFrame", "$parentScrollBar", parent, "FauxScrollFrameTemplate")
		--f:SetPoint("TOPLEFT", 0, -8)
		--f:SetPoint("BOTTOMRIGHT", -30, 8)
		f:SetPoint("TOPLEFT", -7, -1)
		f:SetPoint("BOTTOMRIGHT", -23, 1)

		f:SetScript("OnVerticalScroll", function(self, offset)
			--self:SetValue(offset)
			self.offset = _round(offset / 15)
			_updateScrollBar(parent, tbl)
		end)

		f:SetScript("OnShow", function()
			_updateScrollBar(parent, tbl)
		end)

		return f
	end
end  -- Factory block


-- UI -------------------------------------------------------------------------
-------------------------------------------------------------------------------
do -- UI block
	local uiCreated = false
	function ns:CreateUI()
		if uiCreated then return end

		-- Main Frame
		--Main = self:_frameFactory("Frame", ADDON_NAME.."_UI", UIParent, 320, 500) -- Size of ViewFrame
		Main = self:_frameFactory("Frame", ADDON_NAME.."_UI", UIParent, 320, 580) -- Size of CreateFrame

		Main.title = self:_titleFactory(Main, -5, ADDON_NAME)
		Main.title:SetSize(260, 15)
		Main.title:Disable() -- Don't press it

		Main.bottom = self:_titleFactory(Main, -Main:GetHeight()+20, " ")
		Main.bottom:SetSize(318, 15)
		Main.bottom:Disable() -- Don't press it

		Main.role = self:_iconFactory(Main.title, "TITLE")

		Main.close = self:_frameFactory("Button", "$parent_Close", Main, false, false, "UIPanelCloseButton")
		Main.close:SetPoint("TOPRIGHT", 4, 4)

		-- Tanks frame
		Tanks = self:_frameFactory("Frame", "Tanks", Main, 152, 92)
		Tanks:SetPoint("TOPLEFT", 5, -45)

		Tanks.title = self:_titleFactory(Tanks, 15, L["Tanks"])

		Tanks.role = self:_iconFactory(Tanks.title, "TANK")

		Tanks.scrollBar = self:_scrollBarFactory(Tanks, role["Tanks"], 6) -- Faux
		Tanks.rows = setmetatable({}, { __index = function(t, i)
			local row = self:_slotFactory(i, Tanks)
			
			rawset(t, i, row)
			return row
		end })

		-- Healers frame
		Healers = self:_frameFactory("Frame", "Healers", Main, 152, 92)
		Healers:SetPoint("LEFT", Tanks, "RIGHT", 6, 0)

		Healers.title = self:_titleFactory(Healers, 15, L["Healers"])

		Healers.role = self:_iconFactory(Healers.title, "HEALER")

		Healers.scrollBar = self:_scrollBarFactory(Healers, role["Healers"], 6) -- Faux
		Healers.rows = setmetatable({}, { __index = function(t, i)
			local row = self:_slotFactory(i, Healers)
			
			rawset(t, i, row)
			return row
		end })

		-- Melee frame
		Melee = self:_frameFactory("Frame", "Melee", Main, 152, 152)
		Melee:SetPoint("TOP", Tanks, "BOTTOM", 0, -26)

		Melee.title = self:_titleFactory(Melee, 15, L["Melee"])

		Melee.role = self:_iconFactory(Melee.title, "DPS")

		Melee.scrollBar = self:_scrollBarFactory(Melee, role["Melee"], 10) -- Faux
		Melee.rows = setmetatable({}, { __index = function(t, i)
			local row = self:_slotFactory(i, Melee)
			
			rawset(t, i, row)
			return row
		end })

		-- Ranged frame
		Ranged = self:_frameFactory("Frame", "Ranged", Main, 152, 152)
		Ranged:SetPoint("LEFT", Melee, "RIGHT", 6, 0)

		Ranged.title = self:_titleFactory(Ranged, 15, L["Ranged"])

		Ranged.role = self:_iconFactory(Ranged.title, "DPS")

		Ranged.scrollBar = self:_scrollBarFactory(Ranged, role["Ranged"], 10) -- Faux
		Ranged.rows = setmetatable({}, { __index = function(t, i)
			local row = self:_slotFactory(i, Ranged)
			
			rawset(t, i, row)
			return row
		end })

		-- Signup frame
		Signup = self:_frameFactory("Frame", "Signup", Main, 152, 152)
		Signup:SetPoint("TOP", Melee, "BOTTOM", 0, -26)

		Signup.title = self:_titleFactory(Signup, 15, L["Signups"])

		Signup.role = self:_iconFactory(Signup.title, "SIGNUP")

		Signup.scrollBar = self:_scrollBarFactory(Signup, role["Signup"], 10) -- Faux
		Signup.rows = setmetatable({}, { __index = function(t, i)
			local row = self:_slotFactory(i, Signup)
			
			rawset(t, i, row)
			return row
		end })

		-- Standby frame
		Standby = self:_frameFactory("Frame", "Standby", Main, 152, 92)
		Standby:SetPoint("TOPLEFT", Signup, "TOPRIGHT", 6, 0)

		Standby.title = self:_titleFactory(Standby, 15, L["Standbys"])

		Standby.role = self:_iconFactory(Standby.title, "STANDBY")

		Standby.scrollBar = self:_scrollBarFactory(Standby, role["Standby"], 6) -- Faux
		Standby.rows = setmetatable({}, { __index = function(t, i)
			local row = self:_slotFactory(i, Standby)
			
			rawset(t, i, row)
			return row
		end })

		-- Difficulty Dropdown Menu
		DDDM = self:_frameFactory("Button", ADDON_NAME.."_DDDM", Main, false, false, "UIDropDownMenuTemplate")
		DDDM.selected = defaultDifficulty
		UIDropDownMenu_Initialize(DDDM, _DDDMInitialize)
		UIDropDownMenu_SetWidth(DDDM, 105)
		UIDropDownMenu_JustifyText(DDDM, "CENTER")
		DDDM:SetPoint("TOP", Standby, "BOTTOM", 0, -26)

		DDDM.title = self:_titleFactory(DDDM, 15, L["Raid Difficulty"])
		DDDM.title:SetSize(120, 15)
		DDDM.title:Disable() -- Don't press it

		DDDM.role = self:_iconFactory(DDDM.title, "DIFFICULTY")

		-- Quick and dirty taint fix
		function DDDM:SetSelectedID(id)
		    self.selected = id+13 -- List items 1-4, Difficulty items 14-16
		    UIDropDownMenu_SetText(DDDM, DDDMItems[id]["name"])
		end

		function DDDM:GetSelectedID()
		    return self.selected
		end

		-- Treshold Dropdown Menu
		TDDM = self:_frameFactory("Button", ADDON_NAME.."_TDDM", Main, false, false, "UIDropDownMenuTemplate")
		TDDM.selected = defaultLootTreshold
		UIDropDownMenu_Initialize(TDDM, _TDDMInitialize)
		UIDropDownMenu_SetWidth(TDDM, 105)
		UIDropDownMenu_JustifyText(TDDM, "CENTER")
		TDDM:SetPoint("TOP", DDDM, "BOTTOM", 0, -16)

		TDDM.title = self:_titleFactory(TDDM, 15, L["Loot Settings"])
		TDDM.title:SetSize(120, 15)
		TDDM.title:Disable() -- Don't press it

		TDDM.role = self:_iconFactory(TDDM.title, "LOOT")

		-- Quick and dirty taint fix
		function TDDM:SetSelectedID(id)
		    self.selected = id+1 -- List items 1-4, Treshold items 2-5
		    UIDropDownMenu_SetText(TDDM, ITEM_QUALITY_COLORS[TDDMItems[id]["tres"]].hex..TDDMItems[id]["name"].."|r")
		end

		function TDDM:GetSelectedID()
		    return self.selected
		end

		-- Master Looter Checkbox
		MLCB = self:_frameFactory("CheckButton", ADDON_NAME.."_MLCB", Main, nil, nil, "ChatConfigCheckButtonTemplate")
		MLCB:SetScript("OnClick", _MLCBClick)
		MLCB:SetChecked(defaultLootMethod)
		MLCB:SetPoint("TOPLEFT", TDDM, "BOTTOMLEFT", 20, 5)
		_G[MLCB:GetName().."Text"]:SetText(L["Master Looter"])

		-- Mass Invite Button
		MIB = self:_frameFactory("Button", ADDON_NAME.."_MIB", Main, 125, 28, "UIPanelButtonTemplate")
		MIB:SetScript("OnClick", _MIBClick)
		MIB:SetPoint("TOP", Signup, "BOTTOM", 0, -26)
		MIB:SetText(L["Mass Invite"])

		MIB.title = self:_titleFactory(MIB, 15, L["Mass Invite"])
		MIB.title:SetSize(120, 15)
		MIB.title:Disable() -- Don't press it

		MIB.role = self:_iconFactory(MIB.title, "INVITE")

		uiCreated = true
		Main:Hide()
	end
end -- UI block
-- EOF